searchData={"items":[{"type":"module","title":"NxImage","doc":"Image processing in `Nx`.\n\nAll functions expect images to be tensors in either HWC or CHW order,\nwith an arbitrary number of leading batch axes.\n\nAll transformations preserve the input type, rounding if necessary.\nFor higher precision, cast the input to floating-point beforehand.","ref":"NxImage.html"},{"type":"function","title":"NxImage.center_crop/3","doc":"Crops an image at the center.\n\nIf the image is too small to be cropped to the desired size, it gets\npadded with zeros.","ref":"NxImage.html#center_crop/3"},{"type":"function","title":"Options - NxImage.center_crop/3","doc":"* `:channels` - channels location, either `:first` or `:last`.\n    Defaults to `:last`","ref":"NxImage.html#center_crop/3-options"},{"type":"function","title":"Examples - NxImage.center_crop/3","doc":"iex> image = Nx.iota({4, 4, 1}, type: :u8)\n    iex> NxImage.center_crop(image, {2, 2})\n    #Nx.Tensor \n\n    iex> image = Nx.iota({2, 2, 1}, type: :u8)\n    iex> NxImage.center_crop(image, {1, 4})\n    #Nx.Tensor","ref":"NxImage.html#center_crop/3-examples"},{"type":"function","title":"NxImage.from_continuous/3","doc":"Converts values from continuous range into pixel values (0-255).","ref":"NxImage.html#from_continuous/3"},{"type":"function","title":"Examples - NxImage.from_continuous/3","doc":"iex> image = Nx.tensor([[[0.0], [0.5]], [[0.75], [1.0]]])\n    iex> NxImage.from_continuous(image, 0.0, 1.0)\n    #Nx.Tensor \n\n    iex> image = Nx.tensor([[[-1.0], [0.0]], [[0.5], [1.0]]])\n    iex> NxImage.from_continuous(image, -1.0, 1.0)\n    #Nx.Tensor","ref":"NxImage.html#from_continuous/3-examples"},{"type":"function","title":"NxImage.normalize/4","doc":"Normalizes an image according to the given per-channel mean and\nstandard deviation.\n\n  * `:channels` - channels location, either `:first` or `:last`.\n    Defaults to `:last`","ref":"NxImage.html#normalize/4"},{"type":"function","title":"Examples - NxImage.normalize/4","doc":"iex> image = Nx.iota({2, 2, 3}, type: :f32)\n    iex> mean = Nx.tensor([0.485, 0.456, 0.406])\n    iex> std = Nx.tensor([0.229, 0.224, 0.225])\n    iex> NxImage.normalize(image, mean, std)\n    #Nx.Tensor","ref":"NxImage.html#normalize/4-examples"},{"type":"function","title":"NxImage.resize/3","doc":"Resizes an image.","ref":"NxImage.html#resize/3"},{"type":"function","title":"Options - NxImage.resize/3","doc":"* `:method` - the resizing method to use, either of `:nearest`,\n    `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to\n    `:bilinear`\n\n  * `:antialias` - whether an anti-aliasing filter should be used\n    when downsampling. This has no effect with upsampling. Defaults\n    to `true`\n\n  * `:channels` - channels location, either `:first` or `:last`.\n    Defaults to `:last`","ref":"NxImage.html#resize/3-options"},{"type":"function","title":"Examples - NxImage.resize/3","doc":"iex> image = Nx.iota({2, 2, 1}, type: :u8)\n    iex> NxImage.resize(image, {3, 3}, method: :nearest)\n    #Nx.Tensor \n\n    iex> image = Nx.iota({2, 2, 1}, type: :f32)\n    iex> NxImage.resize(image, {3, 3}, method: :bilinear)\n    #Nx.Tensor","ref":"NxImage.html#resize/3-examples"},{"type":"function","title":"NxImage.resize_short/3","doc":"Scales an image such that the short edge matches the given size.","ref":"NxImage.html#resize_short/3"},{"type":"function","title":"Options - NxImage.resize_short/3","doc":"* `:method` - the resizing method to use, same as `resize/2`\n\n  * `:antialias` - whether an anti-aliasing filter should be used\n    when downsampling. This has no effect with upsampling. Defaults\n    to `true`\n\n  * `:channels` - channels location, either `:first` or `:last`.\n    Defaults to `:last`","ref":"NxImage.html#resize_short/3-options"},{"type":"function","title":"Examples - NxImage.resize_short/3","doc":"iex> image = Nx.iota({2, 4, 1}, type: :u8)\n    iex> resized_image = NxImage.resize_short(image, 3, method: :nearest)\n    iex> Nx.shape(resized_image)\n    {3, 6, 1}\n\n    iex> image = Nx.iota({4, 2, 1}, type: :u8)\n    iex> resized_image = NxImage.resize_short(image, 3, method: :nearest)\n    iex> Nx.shape(resized_image)\n    {6, 3, 1}","ref":"NxImage.html#resize_short/3-examples"},{"type":"function","title":"NxImage.to_continuous/3","doc":"Converts pixel values (0-255) into a continuous range.","ref":"NxImage.html#to_continuous/3"},{"type":"function","title":"Examples - NxImage.to_continuous/3","doc":"iex> image = Nx.tensor([[[0], [128]], [[191], [255]]])\n    iex> NxImage.to_continuous(image, 0.0, 1.0)\n    #Nx.Tensor \n\n    iex> image = Nx.tensor([[[0], [128]], [[191], [255]]])\n    iex> NxImage.to_continuous(image, -1.0, 1.0)\n    #Nx.Tensor","ref":"NxImage.html#to_continuous/3-examples"},{"type":"extras","title":"Examples","doc":"# Examples\n\n```elixir\nMix.install([\n  {:nx, \"~> 0.6.0\"},\n  {:nx_image, \"~> 0.1.2\"},\n  {:kino, \"~> 0.12.0\"}\n])\n```","ref":"examples.html"},{"type":"extras","title":"Upload your test image - Examples","doc":"Using a Livebook v0.8.0 or higher, we can add the new `Kino.Input.image/1` to upload an image to our notebook.\n\n```elixir\nimage_input = Kino.Input.image(\"Uploaded Image\")\n```\n\nWe can use `Input.read/1` to retrieve the information about our image.\n\n```elixir\n%{file_ref: file_ref, format: :rgb, height: height, width: width} = Kino.Input.read(image_input)\n\ncontent = file_ref |> Kino.Input.file_path() |> File.read!()\n```\n\nNxImage requires that the images be tensors in either HWC (default) or CHW\norder, with an arbitrary number of leading batch axes. The input data is\nalready HWC, so creating a tensor is straightforward:\n\n```elixir\nimage_tensor =\n  Nx.from_binary(content, :u8)\n  |> Nx.reshape({height, width, 3})\n```\n\nNow that we have a tensor in the shape of `{height, width, channels}` we operate on it using the `NxImage` module.","ref":"examples.html#upload-your-test-image"},{"type":"extras","title":"Center crop - Examples","doc":"The first capability we'll look at is center cropping.\n\n```elixir\ncenter_cropped_tensor = NxImage.center_crop(image_tensor, {300, 300})\n```\n\nWe've transformed the image from its original size to 300 x 300 by taking the pixels 150 above and below the image center.  Similarly we have the 150 pixels to the left and right of the center point.","ref":"examples.html#center-crop"},{"type":"extras","title":"Visualization - Examples","doc":"Numbers are great, but most of us are visual focused. Let's see what center crop did to our uploaded image:\n\n```elixir\nKino.Layout.grid(\n  [\n    Kino.Image.new(center_cropped_tensor),\n    Kino.Markdown.new(\"**Center of the image**\")\n  ],\n  boxed: true\n)\n```","ref":"examples.html#visualization"},{"type":"extras","title":"Resize - Examples","doc":"We'll resize the image.  Whether this resized image is shrunk or enlarged is dependent upon the original image size.  Resizing to a standard size can be useful when training visual models on a diverse set of source images.\n\n```elixir\nresized_tensor = NxImage.resize(image_tensor, {768, 768}, method: :nearest)\n```\n\nLet's display the original image and the resized image. If you can't tell the difference, try a non-square image, or resizing to a very small resolution instead.\n\n```elixir\noriginal_image = Kino.Image.new(image_tensor)\noriginal_label = Kino.Markdown.new(\"**Original image**\")\n\nresized_image = Kino.Image.new(resized_tensor)\nresized_label = Kino.Markdown.new(\"**Resized image**\")\n\nKino.Layout.grid([\n  Kino.Layout.grid([original_image, original_label], boxed: true),\n  Kino.Layout.grid([resized_image, resized_label], boxed: true)\n])\n```\n\nLet's double check the shape of both images.\n\n```elixir\n{image_tensor.shape, resized_tensor.shape}\n```\n\nWe can see that the resized image has a different shape from the original image shape.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nYou can try other resize strategies: `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. How do they affect the resulting image?","ref":"examples.html#resize"},{"type":"extras","title":"Further exploration - Examples","doc":"There are other functions in the `NxImage` module.\n\nFor example, `NxImage.normalize/3` can be useful for transfer learning, where an original model is further trained on a set of images from your custom domain. The original images had a particular mean and standard deviation. When transfer learning from the base model, your source images are normalized in the same manner as the distribution of the original set of images.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\n> Note: in this notebook we were using the default `Nx.BinaryBackend` for all the operations. To speed up the operations you can configure an optimised backend or compiler, such as `EXLA`.","ref":"examples.html#further-exploration"}],"content_type":"text/markdown"}